@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2009 the original author or authors.
+ * Copyright 2003-2011 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -60,14 +60,16 @@ public class GrabAnnotationTransformation extends ClassCodeVisitorSupport implem
     private static final String GRABRESOLVER_CLASS_NAME = GrabResolver.class.getName();
     private static final String GRAPERESOLVER_DOT_NAME = dotName(GRABRESOLVER_CLASS_NAME);
     private static final String GRABRESOLVER_SHORT_NAME = shortName(GRAPERESOLVER_DOT_NAME);
-    
-    private static final ClassNode THREAD_CLASSNODE = new ClassNode(Thread.class);
+
+    private static final ClassNode THREAD_CLASSNODE = ClassHelper.make(Thread.class);
     private static final List<String> GRABEXCLUDE_REQUIRED = Arrays.asList("group", "module");
     private static final List<String> GRAPERESOLVER_REQUIRED = Arrays.asList("name", "root");
     private static final List<String> GRAB_REQUIRED = Arrays.asList("group", "module", "version");
-    private static final List<String> GRAB_OPTIONAL = Arrays.asList("classifier", "transitive", "conf", "ext");
+    private static final List<String> GRAB_OPTIONAL = Arrays.asList("classifier", "transitive", "conf", "ext", "type", "changing", "force", "initClass");
+    private static final List<String> GRAB_BOOLEAN = Arrays.asList("transitive", "changing", "force", "initClass");
     private static final Collection<String> GRAB_ALL = DefaultGroovyMethods.plus(GRAB_REQUIRED, GRAB_OPTIONAL);
     private static final Pattern IVY_PATTERN = Pattern.compile("([a-zA-Z0-9-/._+=]+)#([a-zA-Z0-9-/._+=]+)(;([a-zA-Z0-9-/.\\(\\)\\[\\]\\{\\}_+=,:@][a-zA-Z0-9-/.\\(\\)\\]\\{\\}_+=,:@]*))?(\\[([a-zA-Z0-9-/._+=,]*)\\])?");
+    private static final Pattern ATTRIBUTES_PATTERN = Pattern.compile("(.*;|^)([a-zA-Z0-9]+)=([a-zA-Z0-9.*\\[\\]\\-\\(\\),]*)$");
 
     private static String dotName(String className) {
         return className.substring(className.lastIndexOf("."));
@@ -96,7 +98,7 @@ public class GrabAnnotationTransformation extends ClassCodeVisitorSupport implem
     boolean allowShortGrabResolver;
     Set<String> grabResolverAliases;
     List<AnnotationNode> grabResolverAnnotations;
-    
+
     SourceUnit sourceUnit;
     ClassLoader loader;
     boolean initContextClassLoader;
@@ -157,10 +159,10 @@ public class GrabAnnotationTransformation extends ClassCodeVisitorSupport implem
             grabConfigAnnotations = new ArrayList<AnnotationNode>();
             grapesAnnotations = new ArrayList<AnnotationNode>();
             grabResolverAnnotations = new ArrayList<AnnotationNode>();
-            
+
             visitClass(classNode);
 
-            ClassNode grapeClassNode = new ClassNode(Grape.class);
+            ClassNode grapeClassNode = ClassHelper.make(Grape.class);
 
             if (!grabResolverAnnotations.isEmpty()) {
                 grabResolverAnnotationLoop:
@@ -201,7 +203,7 @@ public class GrabAnnotationTransformation extends ClassCodeVisitorSupport implem
                     Grape.addResolver(grapeResolverMap);
                 }
             }
-            
+
             if (!grapesAnnotations.isEmpty()) {
                 for (AnnotationNode node : grapesAnnotations) {
                     Expression init = node.getMember("initClass");
@@ -271,7 +273,7 @@ public class GrabAnnotationTransformation extends ClassCodeVisitorSupport implem
                 }
             }
         }
-        
+
         if (!grabMaps.isEmpty()) {
             Map<String, Object> basicArgs = new HashMap<String, Object>();
             basicArgs.put("classLoader", loader != null ? loader : sourceUnit.getClassLoader());
@@ -374,6 +376,30 @@ public class GrabAnnotationTransformation extends ClassCodeVisitorSupport implem
         Object allParts = ((ConstantExpression)val).getValue();
         if (!(allParts instanceof String)) return;
         String allstr = (String) allParts;
+
+        // strip off trailing attributes
+        boolean done = false;
+        while (!done) {
+            Matcher attrs = ATTRIBUTES_PATTERN.matcher(allstr);
+            if (attrs.find()) {
+                String attrName = attrs.group(2);
+                String attrValue = attrs.group(3);
+                if (attrName == null || attrValue == null) continue;
+                boolean isBool = GRAB_BOOLEAN.contains(attrName);
+                ConstantExpression value = new ConstantExpression(isBool ? Boolean.valueOf(attrValue) : attrValue);
+                value.setSourcePosition(node);
+                node.addMember(attrName, value);
+                int lastSemi = allstr.lastIndexOf(';');
+                if (lastSemi == -1) {
+                    allstr = "";
+                    break;
+                }
+                allstr = allstr.substring(0, lastSemi);
+            } else {
+                done = true;
+            }
+        }
+
         if (allstr.contains("#")) {
             // see: http://ant.apache.org/ivy/history/latest-milestone/textual.html
             Matcher m = IVY_PATTERN.matcher(allstr);
@@ -383,7 +409,7 @@ public class GrabAnnotationTransformation extends ClassCodeVisitorSupport implem
             node.addMember("group", new ConstantExpression(m.group(1)));
             if (m.group(6) != null) node.addMember("conf", new ConstantExpression(m.group(6)));
             if (m.group(4) != null) node.addMember("version", new ConstantExpression(m.group(4)));
-            else if (!exclude) node.addMember("version", new ConstantExpression("*"));
+            else if (!exclude && node.getMember("version") == null) node.addMember("version", new ConstantExpression("*"));
             node.getMembers().remove("value");
         } else if (allstr.contains(":")) {
             // assume gradle syntax
@@ -400,7 +426,7 @@ public class GrabAnnotationTransformation extends ClassCodeVisitorSupport implem
             if (parts.length > 4) return;
             if (parts.length > 3) node.addMember("classifier", new ConstantExpression(parts[3]));
             if (parts.length > 2) node.addMember("version", new ConstantExpression(parts[2]));
-            else if (!exclude) node.addMember("version", new ConstantExpression("*"));
+            else if (!exclude && node.getMember("version") == null) node.addMember("version", new ConstantExpression("*"));
             if (ext.length() > 0) node.addMember("ext", new ConstantExpression(ext));
             node.addMember("module", new ConstantExpression(parts[1]));
             node.addMember("group", new ConstantExpression(parts[0]));
@@ -434,7 +460,7 @@ public class GrabAnnotationTransformation extends ClassCodeVisitorSupport implem
                     || (allowShortGrabResolver && GRABRESOLVER_SHORT_NAME.equals(name))
                     || (grabResolverAliases.contains(name))) {
                 grabResolverAnnotations.add(annotation);
-            }            
+            }
         }
     }
 
