@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2009 the original author or authors.
+ * Copyright 2003-2010 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -49,6 +49,7 @@ import java.util.*;
  *
  * @author Jochen Theodorou
  * @author Roshan Dawrani
+ * @author Alex Tkachman
  */
 public class ResolveVisitor extends ClassCodeExpressionTransformer {
     private ClassNode currentClass;
@@ -57,19 +58,18 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
     private CompilationUnit compilationUnit;
     private Map cachedClasses = new HashMap();
     private static final Object NO_CLASS = new Object();
-    private static final Object SCRIPT = new Object();
     private SourceUnit source;
     private VariableScope currentScope;
 
     private boolean isTopLevelProperty = true;
     private boolean inPropertyExpression = false;
     private boolean inClosure = false;
-    private boolean isSpecialConstructorCall = false;
 
     private Map<String, GenericsType> genericParameterNames = new HashMap<String, GenericsType>();
     private Set<FieldNode> fieldTypesChecked = new HashSet<FieldNode>();
     private boolean checkingVariableTypeInDeclaration = false;
     private ImportNode currImportNode = null;
+    private MethodNode currentMethod;
 
     /**
      * we use ConstructedClassWithPackage to limit the resolving the compiler
@@ -168,7 +168,10 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         }
         resolveOrFail(node.getReturnType(), node);
 
+        MethodNode oldCurrentMethod = currentMethod;
+        currentMethod = node;
         super.visitConstructorOrMethod(node, isConstructor);
+        currentMethod = oldCurrentMethod;
 
         genericParameterNames = oldPNames;
         currentScope = oldScope;
@@ -212,7 +215,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
     private boolean resolveToInnerEnum (ClassNode type) {
         // GROOVY-3110: It may be an inner enum defined by this class itself, in which case it does not need to be
         // explicitly qualified by the currentClass name
-    	String name = type.getName();
+        String name = type.getName();
         if(currentClass != type && !name.contains(".") && type.getClass().equals(ClassNode.class)) {
             type.setName(currentClass.getName() + "$" + name);
             if (resolve(type)) return true;
@@ -269,39 +272,37 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
             return true;
         }
 
-        return  resolveNestedClass(type) ||
+        return resolveNestedClass(type) ||
                 resolveFromModule(type, testModuleImports) ||
                 resolveFromCompileUnit(type) ||
                 resolveFromDefaultImports(type, testDefaultImports) ||
                 resolveFromStaticInnerClasses(type, testStaticInnerClasses) ||
-                resolveFromClassCache(type) ||
-                resolveToClass(type) ||
-                resolveToScript(type);
+                resolveToClass(type);
     }
-    
+
     private boolean resolveNestedClass(ClassNode type) {
         // we have for example a class name A, are in class X
         // and there is a nested class A$X. we want to be able 
         // to access that class directly, so A becomes a valid
         // name in X.
-    	// GROOVY-4043: Do this check up the hierarchy, if needed
-    	Map<String, ClassNode> hierClasses = new LinkedHashMap<String, ClassNode>();
-    	ClassNode val;
+        // GROOVY-4043: Do this check up the hierarchy, if needed
+        Map<String, ClassNode> hierClasses = new LinkedHashMap<String, ClassNode>();
+        ClassNode val;
         String name;
-    	for(ClassNode classToCheck = currentClass; classToCheck != ClassHelper.OBJECT_TYPE; 
-    		classToCheck = classToCheck.getSuperClass()) {
-    		if(classToCheck == null || hierClasses.containsKey(classToCheck.getName())) break;
+        for(ClassNode classToCheck = currentClass; classToCheck != ClassHelper.OBJECT_TYPE; 
+            classToCheck = classToCheck.getSuperClass()) {
+            if(classToCheck == null || hierClasses.containsKey(classToCheck.getName())) break;
             hierClasses.put(classToCheck.getName(), classToCheck);
-    	}
-    	
-    	for (ClassNode classToCheck : hierClasses.values()) {
+        }
+        
+        for (ClassNode classToCheck : hierClasses.values()) {
             name = classToCheck.getName()+"$"+type.getName();
             val = ClassHelper.make(name);
             if (resolveFromCompileUnit(val)) {
                 type.setRedirect(val);
                 return true;
             }
-    	}
+        }
         
         // another case we want to check here is if we are in a
         // nested class A$B$C and want to access B without
@@ -338,17 +339,6 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         
         return false;   
     }
-    
-    private boolean resolveFromClassCache(ClassNode type) {
-        String name = type.getName();
-        Object val = cachedClasses.get(name);
-        if (val == null || val == NO_CLASS) {
-            return false;
-        } else {
-            type.setRedirect((ClassNode)val);
-            return true;
-        }
-    }
 
     // NOTE: copied from GroovyClassLoader
     private long getTimeStamp(Class cls) {
@@ -383,17 +373,6 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
     private boolean resolveToScript(ClassNode type) {
         String name = type.getName();
 
-        // We do not need to check instances of LowerCaseClass
-        // to be a script, because unless there was an import for
-        // for this we  do not lookup these cases. This was a decision
-        // made on the mailing list. To ensure we will not visit this
-        // method again we set a NO_CLASS for this name
-        if (type instanceof LowerCaseClass) {
-            cachedClasses.put(name, NO_CLASS);
-        }
-        
-        if (cachedClasses.get(name) == NO_CLASS) return false;
-        if (cachedClasses.get(name) == SCRIPT) cachedClasses.put(name, NO_CLASS);
         if (name.startsWith("java.")) return type.isResolved();
         //TODO: don't ignore inner static classes completely
         if (name.indexOf('$') != -1) return type.isResolved();
@@ -446,21 +425,21 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
             if (type instanceof ConstructedClassWithPackage) {
                 // we replace '.' only in the className part
                 // with '$' to find an inner class. The case that
-                // the package is really a class is handled else where
+                // the package is really a class is handled elsewhere
                 ConstructedClassWithPackage tmp = (ConstructedClassWithPackage) type;
-                String name = ((ConstructedClassWithPackage) type).className;
-                tmp.className = replaceLastPoint(name);
+                String savedName = tmp.className;
+                tmp.className = replaceLastPoint(savedName);
                 if (resolve(tmp, false, true, true)) {
                     type.setRedirect(tmp.redirect());
                     return true;
                 }
-                tmp.className = name;
+                tmp.className = savedName;
             }   else {
-                String name = type.getName();
-                String replacedPointType = replaceLastPoint(name);
+                String savedName = type.getName();
+                String replacedPointType = replaceLastPoint(savedName);
                 type.setName(replacedPointType);
                 if (resolve(type, false, true, true)) return true;
-                type.setName(name);
+                type.setName(savedName);
             }
         }
         return false;
@@ -470,7 +449,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         // test default imports
         testDefaultImports &= !type.hasPackageName();
         // we do not resolve a vanilla name starting with a lower case letter
-        // try to resolve against adefault import, because we know that the
+        // try to resolve against a default import, because we know that the
         // default packages do not contain classes like these
         testDefaultImports &= !(type instanceof LowerCaseClass);
         if (testDefaultImports) {
@@ -480,8 +459,8 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
                 // We limit the inner class lookups here by using ConstructedClassWithPackage.
                 // This way only the name will change, the packagePrefix will
                 // not be included in the lookup. The case where the
-                // packagePrefix is really a class is handled else where.
-                // WARNING: This code does not expect a class that has an static
+                // packagePrefix is really a class is handled elsewhere.
+                // WARNING: This code does not expect a class that has a static
                 //          inner class in DEFAULT_IMPORTS
                 ConstructedClassWithPackage tmp =  new ConstructedClassWithPackage(packagePrefix,name);
                 if (resolve(tmp, false, false, false)) {
@@ -524,7 +503,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
     private boolean resolveAliasFromModule(ClassNode type) {
         // In case of getting a ConstructedClassWithPackage here we do not do checks for partial
         // matches with imported classes. The ConstructedClassWithPackage is already a constructed
-        // node and any subclass resolving will then take elsewhere place
+        // node and any subclass resolving will then take place elsewhere
         if (type instanceof ConstructedClassWithPackage) return false;
 
         ModuleNode module = currentClass.getModule();
@@ -545,16 +524,29 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
             pname = name.substring(0, index);
             ClassNode aliasedNode = null;
             ImportNode importNode = module.getImport(pname);
-            if(importNode != null && importNode != currImportNode) {
+            if (importNode != null && importNode != currImportNode) {
                 aliasedNode = importNode.getType();
             }
+            if (aliasedNode == null) {
+                importNode = module.getStaticImports().get(pname);
+                if (importNode != null && importNode != currImportNode) {
+                    // static alias only for inner classes and must be at end of chain
+                    ClassNode tmp = ClassHelper.make(importNode.getType().getName() + "$" + importNode.getFieldName());
+                    if (resolve(tmp, false, false, true)) {
+                        if ((tmp.getModifiers() & Opcodes.ACC_STATIC) != 0) {
+                            type.setRedirect(tmp.redirect());
+                            return true;
+                        }
+                    }
+                }
+            }
 
             if (aliasedNode != null) {
                 if (pname.length() == name.length()) {
                     // full match
 
                     // We can compare here by length, because pname is always
-                    // a sbustring of name, so same length means they are equal.
+                    // a substring of name, so same length means they are equal.
                     type.setRedirect(aliasedNode);
                     return true;
                 } else {
@@ -567,7 +559,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
                     // Since we do not want to have useless lookups we create the name
                     // completely and use a ConstructedClassWithPackage to prevent lookups against the package.
                     String className = aliasedNode.getNameWithoutPackage() + '$' +
-                                       name.substring(pname.length()+1).replace('.', '$');
+                            name.substring(pname.length() + 1).replace('.', '$');
                     ConstructedClassWithPackage tmp = new ConstructedClassWithPackage(aliasedNode.getPackageName()+".", className);
                     if (resolve(tmp, true, true, false)) {
                         type.setRedirect(tmp.redirect());
@@ -624,20 +616,34 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
                 // check package this class is defined in. The usage of ConstructedClassWithPackage here
                 // means, that the module package will not be involved when the
                 // compiler tries to find an inner class.
-                ConstructedClassWithPackage tmp =  new ConstructedClassWithPackage(module.getPackageName(),name);
+                ConstructedClassWithPackage tmp =  new ConstructedClassWithPackage(module.getPackageName(), name);
                 if (resolve(tmp, false, false, false)) {
+                    ambiguousClass(type, tmp, name);
                     type.setRedirect(tmp.redirect());
                     return true;
                 }
             }
 
-            // check module node imports packages
+            // check module static imports (for static inner classes)
+            for (ImportNode importNode : module.getStaticImports().values()) {
+                if (importNode.getFieldName().equals(name)) {
+                    ClassNode tmp = ClassHelper.make(importNode.getType().getName() + "$" + name);
+                    if (resolve(tmp, false, false, true)) {
+                        if ((tmp.getModifiers() & Opcodes.ACC_STATIC) != 0) {
+                            type.setRedirect(tmp.redirect());
+                            return true;
+                        }
+                    }
+                }
+            }
+
+            // check module node import packages
             for (ImportNode importNode : module.getStarImports()) {
                 String packagePrefix = importNode.getPackageName();
                 // We limit the inner class lookups here by using ConstructedClassWithPackage.
                 // This way only the name will change, the packagePrefix will
                 // not be included in the lookup. The case where the
-                // packagePrefix is really a class is handled else where.
+                // packagePrefix is really a class is handled elsewhere.
                 ConstructedClassWithPackage tmp = new ConstructedClassWithPackage(packagePrefix, name);
                 if (resolve(tmp, false, false, true)) {
                     ambiguousClass(type, tmp, name);
@@ -645,6 +651,19 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
                     return true;
                 }
             }
+
+            // check for star imports (import static pkg.Outer.*) matching static inner classes
+            for (ImportNode importNode : module.getStaticStarImports().values()) {
+                ClassNode tmp = ClassHelper.make(importNode.getClassName() + "$" + name);
+                if (resolve(tmp, false, false, true)) {
+                    if ((tmp.getModifiers() & Opcodes.ACC_STATIC) != 0) {
+                        ambiguousClass(type, tmp, name);
+                        type.setRedirect(tmp.redirect());
+                        return true;
+                    }
+                }
+
+            }
         }
         return false;
     }
@@ -652,15 +671,6 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
     private boolean resolveToClass(ClassNode type) {
         String name = type.getName();
 
-        // We do not need to check instances of LowerCaseClass
-        // to be a Class, because unless there was an import for
-        // for this we  do not lookup these cases. This was a decision
-        // made on the mailing list. To ensure we will not visit this
-        // method again we set a NO_CLASS for this name
-        if (type instanceof LowerCaseClass) {
-            cachedClasses.put(name,NO_CLASS);
-        }
-
         // We use here the class cache cachedClasses to prevent
         // calls to ClassLoader#loadClass. disabling this cache will
         // cause a major performance hit. Unlike at the end of this
@@ -668,11 +678,27 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         // want to recompile or not. If the class was cached, then
         // we do not want to recompile, recompilation is already
         // scheduled then
+
         Object cached = cachedClasses.get(name);
-        if (cached == NO_CLASS) return false;
-        // cached == SCRIPT should not happen here!
-        if (cached == SCRIPT) throw new GroovyBugError("name "+name+" was marked as script, but was not resolved as such");
-        if (cached != null) return true;
+        if (cached == NO_CLASS)
+            return false;
+
+        if (cached != null) {
+            // cached == SCRIPT should not happen here!
+            type.setRedirect((ClassNode) cached);
+            return true;
+        }
+
+
+        // We do not need to check instances of LowerCaseClass
+        // to be a Class, because unless there was an import for
+        // for this we  do not lookup these cases. This was a decision
+        // made on the mailing list. To ensure we will not visit this
+        // method again we set a NO_CLASS for this name
+        if (type instanceof LowerCaseClass) {
+            cachedClasses.put(name,NO_CLASS);
+            return false;
+        }
 
         if (currentClass.getModule().hasPackageName() && name.indexOf('.') == -1) return false;
         GroovyClassLoader loader = compilationUnit.getClassLoader();
@@ -682,16 +708,16 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
             // here since the GroovyClassLoader would create a new CompilationUnit
             cls = loader.loadClass(name, false, true);
         } catch (ClassNotFoundException cnfe) {
-            cachedClasses.put(name, SCRIPT);
-            return false;
+            cachedClasses.put(name, NO_CLASS);
+            return resolveToScript(type);
         } catch (CompilationFailedException cfe) {
             compilationUnit.getErrorCollector().addErrorAndContinue(new ExceptionMessage(cfe, true, source));
-            return false;
+            return resolveToScript(type);
         }
         //TODO: the case of a NoClassDefFoundError needs a bit more research
         // a simple recompilation is not possible it seems. The current class
         // we are searching for is there, so we should mark that somehow.
-        // Basically the missing class needs to be completly compiled before
+        // Basically the missing class needs to be completely compiled before
         // we can again search for the current name.
         /*catch (NoClassDefFoundError ncdfe) {
             cachedClasses.put(name,SCRIPT);
@@ -705,7 +731,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         //      because  we want to give a possible script a chance to
         //      recompile. This can only be done if the loader was not
         //      the instance defining the class.
-        return cls.getClassLoader() == loader;
+        return cls.getClassLoader() == loader || resolveToScript(type);
     }
 
 
@@ -873,56 +899,45 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         if (isTopLevelProperty) ret = correctClassClassChain(pe);
         return ret;
     }
-    
-    
+
     private void checkThisAndSuperAsPropertyAccess(PropertyExpression expression) {
         if (expression.isImplicitThis()) return;
         String prop = expression.getPropertyAsString();
-        if (prop==null) return;
+        if (prop == null) return;
         if (!prop.equals("this") && !prop.equals("super")) return;
-        
-        //TODO: add support for super
-        if (prop.equals("super")) {
-            addError("Inner classes referencing outer classes using super is not supported yet.",expression);
-        }
-        
-        if (!(expression.getObjectExpression() instanceof ClassExpression)) {
-            addError("The usage of '.this' or '.super' requires an explicit class in front.",expression);
-            return;
-        }
-            
-        if (!(currentClass instanceof InnerClassNode)) {
-            addError("The usage of '.this' and '.super' is only allowed in a inner class",expression);
-            return;
-        }
 
-        
-        ClassNode type  = expression.getObjectExpression().getType();
-        ClassNode iterType = currentClass;
-        while (iterType!=null) {
-            if (iterType.equals(type)) break;
-            iterType = iterType.getOuterClass();
+        if (expression.getObjectExpression() instanceof ClassExpression) {
+            if (!(currentClass instanceof InnerClassNode)) {
+                addError("The usage of 'Class.this' and 'Class.super' is only allowed in nested/inner classes.", expression);
+                return;
+            }
+            ClassNode type = expression.getObjectExpression().getType();
+            ClassNode iterType = currentClass;
+            while (iterType != null) {
+                if (iterType.equals(type)) break;
+                iterType = iterType.getOuterClass();
+            }
+            if (iterType == null) {
+                addError("The class '" + type.getName() + "' needs to be an outer class of '" +
+                        currentClass.getName() + "' when using '.this' or '.super'.", expression);
+            }
+            if ((currentClass.getModifiers() & Opcodes.ACC_STATIC) == 0) return;
+            if (!currentScope.isInStaticContext()) return;
+            addError("The usage of 'Class.this' and 'Class.super' within static nested class '" +
+                    currentClass.getName() + "' is not allowed in a static context.", expression);
         }
-        if (iterType==null) {
-            addError("The class '"+type.getName()+"' needs to be an "+
-                     "outer class of '"+currentClass.getName()+"'.",expression);
-        }
-
-        
-        if ((currentClass.getModifiers() & Opcodes.ACC_STATIC)==0) return;
-        if (!currentScope.isInStaticContext()) return;
-        addError("The usage of '.this' and '.super' is only in nonstatic context",expression);
     }
 
     protected Expression transformVariableExpression(VariableExpression ve) {
+        visitAnnotations(ve);
         Variable v = ve.getAccessedVariable();
         
         if(!(v instanceof DynamicVariable) && !checkingVariableTypeInDeclaration) {
-        	/*
-        	 *  GROOVY-4009: when a normal variable is simply being used, there is no need to try to 
-        	 *  resolve its type. Variable type resolve should proceed only if the variable is being declared. 
-        	 */
-        	return ve;
+            /*
+             *  GROOVY-4009: when a normal variable is simply being used, there is no need to try to 
+             *  resolve its type. Variable type resolve should proceed only if the variable is being declared. 
+             */
+            return ve;
         }
         if (v instanceof DynamicVariable){
             String name = ve.getName();
@@ -1000,7 +1015,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
                     if (map) {
                         final MapExpression me = new MapExpression();
                         for (Expression expression : list.getExpressions()) {
-                            me.addMapEntryExpression((MapEntryExpression) expression);
+                            me.addMapEntryExpression((MapEntryExpression) transform(expression));
                         }
                         me.setSourcePosition(list);
                         final CastExpression ce = new CastExpression(left.getType(), me);
@@ -1013,7 +1028,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
             if (be.getRightExpression() instanceof MapEntryExpression) {
                 // may be we have C[k1:v1] -> should become (C)([k1:v1])
                 final MapExpression me = new MapExpression();
-                me.addMapEntryExpression((MapEntryExpression) be.getRightExpression());
+                me.addMapEntryExpression((MapEntryExpression) transform(be.getRightExpression()));
                 me.setSourcePosition(be.getRightExpression());
                 final CastExpression ce = new CastExpression(left.getType(), me);
                 ce.setSourcePosition(be);
@@ -1034,12 +1049,14 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
             for (Parameter para : paras) {
                 ClassNode t = para.getType();
                 resolveOrFail(t, ce);
+                visitAnnotations(para);
                 if (para.hasInitialExpression()) {
                     Object initialVal = para.getInitialExpression();
                     if (initialVal instanceof Expression) {
-                    	para.setInitialExpression(transform((Expression) initialVal));
+                        para.setInitialExpression(transform((Expression) initialVal));
                     }
                 }
+                visitAnnotations(para);
             }
         }
         Statement code = ce.getCode();
@@ -1054,9 +1071,8 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         if (Modifier.isAbstract(type.getModifiers())) {
             addError("You cannot create an instance from the abstract " + getDescription(type) + ".", cce);
         }
-        isSpecialConstructorCall = cce.isSpecialCall();
+
         Expression ret = cce.transformExpression(this);
-        isSpecialConstructorCall = false;
         return ret;
     }
 
@@ -1069,15 +1085,20 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         Expression method = transform(mce.getMethod());
         Expression object = transform(mce.getObjectExpression());
 
+        resolveGenericsTypes(mce.getGenericsTypes());
+        
         MethodCallExpression result = new MethodCallExpression(object, method, args);
         result.setSafe(mce.isSafe());
         result.setImplicitThis(mce.isImplicitThis());
         result.setSpreadSafe(mce.isSpreadSafe());
         result.setSourcePosition(mce);
+        result.setGenericsTypes(mce.getGenericsTypes());
+        result.setMethodTarget(mce.getMethodTarget());
         return result;
     }
     
     protected Expression transformDeclarationExpression(DeclarationExpression de) {
+        visitAnnotations(de);
         Expression oldLeft = de.getLeftExpression();
         checkingVariableTypeInDeclaration = true;
         Expression left = transform(oldLeft);
@@ -1088,12 +1109,25 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
             return de;
         }
         Expression right = transform(de.getRightExpression());
-        if (right == de.getRightExpression()) return de;
+        if (right == de.getRightExpression()) {
+            fixDeclaringClass(de);
+            return de;
+        }
         DeclarationExpression newDeclExpr = new DeclarationExpression(left, de.getOperation(), right);
+        newDeclExpr.setDeclaringClass(de.getDeclaringClass());
+        fixDeclaringClass(newDeclExpr);
         newDeclExpr.setSourcePosition(de);
+        newDeclExpr.addAnnotations(de.getAnnotations());
         return newDeclExpr;
     }
 
+    // TODO get normal resolving to set declaring class
+    private void fixDeclaringClass(DeclarationExpression newDeclExpr) {
+        if (newDeclExpr.getDeclaringClass() == null && currentMethod != null) {
+            newDeclExpr.setDeclaringClass(currentMethod.getDeclaringClass());
+        }
+    }
+
     protected Expression transformAnnotationConstantExpression(AnnotationConstantExpression ace) {
         AnnotationNode an = (AnnotationNode) ace.getValue();
         ClassNode type = an.getClassNode();
@@ -1121,14 +1155,14 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
                 checkAnnotationMemberValue(newValue);
             }
             if(annType.isResolved()) {
-            	Class annTypeClass = annType.getTypeClass();
-            	Retention retAnn = (Retention) annTypeClass.getAnnotation(Retention.class);
-            	if (retAnn != null && retAnn.value().equals(RetentionPolicy.RUNTIME)) {
-            		AnnotationNode anyPrevAnnNode = tmpAnnotations.put(annTypeClass.getName(), an);
-            		if(anyPrevAnnNode != null) {
-            			addError("Cannot specify duplicate annotation on the same member : " + annType.getName(), an);
-            		}
-            	}
+                Class annTypeClass = annType.getTypeClass();
+                Retention retAnn = (Retention) annTypeClass.getAnnotation(Retention.class);
+                if (retAnn != null && retAnn.value().equals(RetentionPolicy.RUNTIME)) {
+                    AnnotationNode anyPrevAnnNode = tmpAnnotations.put(annTypeClass.getName(), an);
+                    if(anyPrevAnnNode != null) {
+                        addError("Cannot specify duplicate annotation on the same member : " + annType.getName(), an);
+                    }
+                }
             }
         }
     }
@@ -1188,19 +1222,25 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
 
     public void visitClass(ClassNode node) {
         ClassNode oldNode = currentClass;
-        currentClass = node;
 
+        if (node instanceof InnerClassNode) {
+            if (Modifier.isStatic(node.getModifiers())) {
+                genericParameterNames = new HashMap<String, GenericsType>();
+            }
+        } else {
+            genericParameterNames = new HashMap<String, GenericsType>();
+        }
+        currentClass = node;
         resolveGenericsHeader(node.getGenericsTypes());
 
         ModuleNode module = node.getModule();
         if (!module.hasImportsResolved()) {
-            List l = module.getImports();
             for (ImportNode importNode : module.getImports()) {
-            	currImportNode = importNode;
+                currImportNode = importNode;
                 ClassNode type = importNode.getType();
                 if (resolve(type, false, false, true)) {
-                	currImportNode = null;
-                	continue;
+                    currImportNode = null;
+                    continue;
                 }
                 currImportNode = null;
                 addError("unable to resolve class " + type.getName(), type);
@@ -1208,14 +1248,14 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
             for (ImportNode importNode : module.getStaticStarImports().values()) {
                 ClassNode type = importNode.getType();
                 if (resolve(type, false, false, true)) continue;
-                // May be this type belongs in the same package as the node that is doing the
+                // Maybe this type belongs in the same package as the node that is doing the
                 // static import. In that case, the package may not have been explicitly specified.
                 // Try with the node's package too. If still not found, revert to original type name.
                 if (type.getPackageName() == null && node.getPackageName() != null) {
-                	String oldTypeName = type.getName();
-                	type.setName(node.getPackageName() + "." + oldTypeName);
-                	if (resolve(type, false, false, true)) continue;
-                	type.setName(oldTypeName);
+                    String oldTypeName = type.getName();
+                    type.setName(node.getPackageName() + "." + oldTypeName);
+                    if (resolve(type, false, false, true)) continue;
+                    type.setName(oldTypeName);
                 }
                 addError("unable to resolve class " + type.getName(), type);
             }
