@@ -52,7 +52,7 @@ import com.thoughtworks.xstream.XStream;
  *
  * @author <a href="mailto:cpoirier@dreaming.org">Chris Poirier</a>
  * @author <a href="mailto:b55r@sina.com">Bing Ran</a>
- * @version $Id: SourceUnit.java 21480 2011-02-04 13:55:32Z blackdrag $
+ * @version $Id$
  */
 
 public class SourceUnit extends ProcessingUnit {
@@ -270,10 +270,13 @@ public class SourceUnit extends ProcessingUnit {
 
         try {
             this.ast = parserPlugin.buildAST(this, this.classLoader, this.cst);
-
             this.ast.setDescription(this.name);
         }
         catch (SyntaxException e) {
+            if (this.ast == null) {
+                // Create a dummy ModuleNode to represent a failed parse - in case a later phase attempts to use the ast
+                this.ast = new ModuleNode(this);
+            }
             getErrorCollector().addError(new SyntaxErrorMessage(e, this));
         }
 
@@ -358,4 +361,3 @@ public class SourceUnit extends ProcessingUnit {
 
     public ReaderSource getSource() { return source; }
 }
-
