@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2010 the original author or authors.
+ * Copyright 2003-2012 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -26,10 +26,21 @@ public class SimpleGroovyClassDoc extends SimpleGroovyAbstractableElementDoc imp
     public static final Pattern TAG_REGEX = Pattern.compile("(?sm)\\s*@([a-zA-Z.]+)\\s+(.*?)(?=\\s+@)");
     public static final Pattern LINK_REGEX = Pattern.compile("(?m)[{]@(link)\\s+([^}]*)}");
     public static final Pattern CODE_REGEX = Pattern.compile("(?m)[{]@(code)\\s+([^}]*)}");
-    public static final Pattern REF_LABEL_REGEX = Pattern.compile("([\\w.#]*(\\(.*\\))?)(\\s(.*))?");
+    public static final Pattern REF_LABEL_REGEX = Pattern.compile("([\\w.#\\$]*(\\(.*\\))?)(\\s(.*))?");
     public static final Pattern NAME_ARGS_REGEX = Pattern.compile("([^(]+)\\(([^)]*)\\)");
     public static final Pattern SPLIT_ARGS_REGEX = Pattern.compile(",\\s*");
     private static final List<String> PRIMITIVES = Arrays.asList("void", "boolean", "byte", "short", "char", "int", "long", "float", "double");
+    private static final Map<String, String> TAG_TEXT = new HashMap<String, String>();
+    static {
+        TAG_TEXT.put("see", "See Also");
+        TAG_TEXT.put("param", "Parameters");
+        TAG_TEXT.put("throw", "Throws");
+        TAG_TEXT.put("exception", "Throws");
+        TAG_TEXT.put("return", "Returns");
+        TAG_TEXT.put("since", "Since");
+        TAG_TEXT.put("author", "Authors");
+        TAG_TEXT.put("version", "Version");
+    }
     private final List<GroovyConstructorDoc> constructors;
     private final List<GroovyFieldDoc> fields;
     private final List<GroovyFieldDoc> properties;
@@ -44,6 +55,7 @@ public class SimpleGroovyClassDoc extends SimpleGroovyAbstractableElementDoc imp
     private GroovyClassDoc outer;
     private String superClassName;
     private String fullPathName;
+    private boolean isgroovy;
     private GroovyRootDoc savedRootDoc = null;
 
     public SimpleGroovyClassDoc(List<String> importedClassesAndPackages, String name, List<LinkArgument> links) {
@@ -85,6 +97,14 @@ public class SimpleGroovyClassDoc extends SimpleGroovyAbstractableElementDoc imp
         this.outer = outer;
     }
 
+    public boolean isGroovy() {
+        return isgroovy;
+    }
+
+    public void setGroovy(boolean isgroovy) {
+        this.isgroovy = isgroovy;
+    }
+
     /**
      * returns a sorted array of nested classes and interfaces
      */
@@ -171,7 +191,7 @@ public class SimpleGroovyClassDoc extends SimpleGroovyAbstractableElementDoc imp
 
     public String getRelativeRootPath() {
         StringTokenizer tokenizer = new StringTokenizer(fullPathName, "/"); // todo windows??
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
         if (tokenizer.hasMoreTokens()) {
             tokenizer.nextToken(); // ignore the first token, as we want n-1 parent dirs
         }
@@ -334,7 +354,7 @@ public class SimpleGroovyClassDoc extends SimpleGroovyAbstractableElementDoc imp
     }
 
     private static String resolveMethodArgs(GroovyRootDoc rootDoc, SimpleGroovyClassDoc classDoc, String type) {
-        if (type.indexOf("(") < 0) return type;
+        if (!type.contains("(")) return type;
         Matcher m = NAME_ARGS_REGEX.matcher(type);
         if (m.matches()) {
             String name = m.group(1);
@@ -393,7 +413,7 @@ public class SimpleGroovyClassDoc extends SimpleGroovyAbstractableElementDoc imp
         final String[] target = type.split("#");
         String shortClassName = target[0].replaceAll(".*\\.", "");
         shortClassName += (target.length > 1 ? "#" + target[1].split("\\(")[0] : "");
-        String name = (full ? target[0] : shortClassName).replaceAll("#", ".");
+        String name = (full ? target[0] : shortClassName).replaceAll("#", ".").replace('$', '.');
 
         // last chance lookup for classes within the current codebase
         if (rootDoc != null) {
@@ -417,15 +437,23 @@ public class SimpleGroovyClassDoc extends SimpleGroovyAbstractableElementDoc imp
     }
 
     private static String buildUrl(String relativeRoot, String[] target, String shortClassName) {
-        if (!relativeRoot.endsWith("/")) {
+        if (relativeRoot.length() > 0 && !relativeRoot.endsWith("/")) {
             relativeRoot += "/";
         }
-        String url = relativeRoot + target[0].replace('.', '/') + ".html" + (target.length > 1 ? "#" + target[1] : "");
+        String url = relativeRoot + target[0].replace('.', '/').replace('$', '.') + ".html" + (target.length > 1 ? "#" + target[1] : "");
         return "<a href='" + url + "' title='" + shortClassName + "'>" + shortClassName + "</a>";
     }
 
     private GroovyClassDoc resolveClass(GroovyRootDoc rootDoc, String name) {
         if (isPrimitiveType(name)) return null;
+        if (name.endsWith("[]")) {
+            GroovyClassDoc componentClass = resolveClass(rootDoc, name.substring(0, name.length() - 2));
+            if (componentClass != null) return new ArrayClassDocWrapper(componentClass);
+            return null;
+        }
+        if (name.equals("T") || name.equals("U") || name.equals("K") || name.equals("V") || name.equals("G")) {
+            name = "java/lang/Object";
+        }
         GroovyClassDoc doc = ((SimpleGroovyRootDoc)rootDoc).classNamedExact(name);
         if (doc != null) return doc;
         int slashIndex = name.lastIndexOf("/");
@@ -442,23 +470,55 @@ public class SimpleGroovyClassDoc extends SimpleGroovyAbstractableElementDoc imp
 
         // The class is not in the tree being documented
         String shortname = name;
-        Class c = null;
+        Class c;
         if (slashIndex > 0) {
             shortname = name.substring(slashIndex + 1);
             c = resolveExternalFullyQualifiedClass(name);
         } else {
             c = resolveExternalClassFromImport(name);
         }
+        if (c == null) {
+            c = resolveFromJavaLang(name);
+        }
         if (c != null) {
             return new ExternalGroovyClassDoc(c);
         }
 
+        if (name.contains("/")) {
+            // search for nested class
+            if (slashIndex > 0) {
+                String outerName = name.substring(0, slashIndex);
+                GroovyClassDoc gcd = resolveClass(rootDoc, outerName);
+                if (gcd instanceof ExternalGroovyClassDoc) {
+                    ExternalGroovyClassDoc egcd = (ExternalGroovyClassDoc) gcd;
+                    String innerName = name.substring(slashIndex+1);
+                    Class outerClass = egcd.externalClass();
+                    for (Class inner : outerClass.getDeclaredClasses()) {
+                        if (inner.getName().equals(outerClass.getName() + "$" + innerName)) {
+                            return new ExternalGroovyClassDoc(inner);
+                        }
+                    }
+                }
+            }
+        }
+
         // and we can't find it
         SimpleGroovyClassDoc placeholder = new SimpleGroovyClassDoc(null, shortname);
         placeholder.setFullPathName(name);
         return placeholder;
     }
 
+    private Class resolveFromJavaLang(String name) {
+        try {
+            return Class.forName("java.lang." + name);
+        } catch (NoClassDefFoundError e) {
+            // ignore
+        } catch (ClassNotFoundException e) {
+            // ignore
+        }
+        return null;
+    }
+
     private static boolean isPrimitiveType(String name) {
         String type = name;
         if (name.endsWith("[]")) type = name.substring(0, name.length() - 2);
@@ -640,25 +700,29 @@ public class SimpleGroovyClassDoc extends SimpleGroovyAbstractableElementDoc imp
         result = replaceAllTags(result, "<TT>", "</TT>", CODE_REGEX);
 
         // hack to reformat other groovydoc block tags (@see, @return, @param, @throws, @author, @since) into html
-        result = replaceAllTags(result + "@endMarker", "<DL><DT><B>$1:</B></DT><DD>", "</DD></DL>", TAG_REGEX);
-        // remove @endMarker
-        result = result.substring(0, result.length() - 10);
+        result = replaceAllTagsCollated(result, "<DL><DT><B>", ":</B></DT><DD>", "</DD><DD>", "</DD></DL>", TAG_REGEX);
 
         return decodeSpecialSymbols(result);
     }
 
-    // TODO: this should go away once we have proper tags
     public String replaceAllTags(String self, String s1, String s2, Pattern regex) {
+        return replaceAllTags(self, s1, s2, regex, links, getRelativeRootPath(), savedRootDoc, this);
+    }
+
+    // TODO: this should go away once we have proper tags
+    public static String replaceAllTags(String self, String s1, String s2, Pattern regex, List<LinkArgument> links, String relPath, GroovyRootDoc rootDoc, SimpleGroovyClassDoc classDoc) {
         Matcher matcher = regex.matcher(self);
         if (matcher.find()) {
             matcher.reset();
             StringBuffer sb = new StringBuffer();
             while (matcher.find()) {
                 String tagname = matcher.group(1);
-                if (tagname.equals("see") || tagname.equals("link")) {
-                    matcher.appendReplacement(sb, s1 + getDocUrl(encodeSpecialSymbols(matcher.group(2))) + s2);
-                } else if (!tagname.equals("interface")) {
-                    matcher.appendReplacement(sb, s1 + encodeSpecialSymbols(matcher.group(2)) + s2);
+                if (!"interface".equals(tagname)) {
+                    String content = encodeSpecialSymbols(matcher.group(2));
+                    if ("link".equals(tagname) || "see".equals(tagname)) {
+                        content = getDocUrl(content, false, links, relPath, rootDoc, classDoc);
+                    }
+                    matcher.appendReplacement(sb, s1 + content + s2);
                 }
             }
             matcher.appendTail(sb);
@@ -668,6 +732,56 @@ public class SimpleGroovyClassDoc extends SimpleGroovyAbstractableElementDoc imp
         }
     }
 
+    // TODO: is there a better way to do this?
+    public String replaceAllTagsCollated(String self, String preKey, String postKey,
+                                         String valueSeparator, String postValues, Pattern regex) {
+        Matcher matcher = regex.matcher(self + "@endMarker");
+        if (matcher.find()) {
+            matcher.reset();
+            Map<String, List<String>> savedTags = new HashMap<String, List<String>>();
+            StringBuffer sb = new StringBuffer();
+            while (matcher.find()) {
+                String tagname = matcher.group(1);
+                if (!"interface".equals(tagname)) {
+                    String content = encodeSpecialSymbols(matcher.group(2));
+                    if ("see".equals(tagname) || "link".equals(tagname)) {
+                        content = getDocUrl(content);
+                    } else if ("param".equals(tagname)) {
+                        int index = content.indexOf(" ");
+                        if (index >= 0) {
+                            content = "<code>" + content.substring(0, index) + "</code> - " + content.substring(index);
+                        }
+                    }
+                    if (TAG_TEXT.containsKey(tagname)) {
+                        String text = TAG_TEXT.get(tagname);
+                        List<String> contents = savedTags.get(text);
+                        if (contents == null) {
+                            contents = new ArrayList<String>();
+                            savedTags.put(text, contents);
+                        }
+                        contents.add(content);
+                        matcher.appendReplacement(sb, "");
+                    } else {
+                        matcher.appendReplacement(sb, preKey + tagname + postKey + content + postValues);
+                    }
+                }
+            }
+            matcher.appendTail(sb);
+            // remove @endMarker
+            sb = new StringBuffer(sb.substring(0, sb.length() - 10));
+            for (Map.Entry<String, List<String>> e : savedTags.entrySet()) {
+                sb.append(preKey);
+                sb.append(e.getKey());
+                sb.append(postKey);
+                sb.append(DefaultGroovyMethods.join(e.getValue(), valueSeparator));
+                sb.append(postValues);
+            }
+            return sb.toString();
+        } else {
+            return self;
+        }
+    }
+
     public static String encodeSpecialSymbols(String text) {
         return Matcher.quoteReplacement(text.replaceAll("@", "&at;"));
     }
