@@ -1,3 +1,18 @@
+/*
+ * Copyright 2003-2011 the original author or authors.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 package org.codehaus.groovy.ast;
 
 import java.io.FileNotFoundException;
@@ -19,7 +34,7 @@ import junit.framework.TestSuite;
  * The section above can be repeated for every new TestCase
  * 
  * The AST output from the visitor is quite big. Also for small source code snippets. Therefore
- * it is possible to only specify the nodes that you wan't to check and separate them with a semicolon.
+ * it is possible to only specify the nodes that you want to check and separate them with a semicolon.
  * A semicolon is also needed when you begin with a new line.
  * Bsp: 
  * [TryCatchStatement,(1:1),(9:2)][BlockStatement,(1:5),(3:3)];
@@ -66,7 +81,7 @@ public class LineColumnCheckTestSuite extends TestSuite {
 
     private static String getContent(Reader reader) {
         int character;
-        StringBuffer content = new StringBuffer();
+        StringBuilder content = new StringBuilder();
         try {
             while ((character = reader.read()) != -1) {
                 content.append((char) character);
